#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "gop_list.h"
#include "filename.h"
#include "registry.h"
#include "gl_dialog.h"
#include "video_stream.h"
#include "sequence_header.h"
#include "picture_header.h"

static int is_registered_suffix(char *filepath)
{
	int i,j,n;
	char *sfx;
	char *trg;
	
	static char *registered_suffix[] = {
		".mpeg",
		".mpg",
		".m2p",
		".mp2",
		".vob",
		".vro",
		".m2v",
		".m1v",
		".mpv",
		".ves",
		".m2t",
		".ssg",
		".ts",
		".bs",
		".m2ts",
	};

	sfx = read_suffix(filepath);
	n = sizeof(registered_suffix) / sizeof(registered_suffix[0]);
	for (i=0;i<n;i++) {
		trg = registered_suffix[i];
		j = 0;
		while ( sfx[j] && trg[j] ) {
			if ( toupper(sfx[j]) != toupper(trg[j]) ) {
				break;
			}
			j += 1;
		}
		if ((sfx[j] == 0) && (trg[j] == 0)) {
			return 1;
		}
	}

	return 0;
}

static void make_gop_list(char *src_path)
{
	int code;
	int orig_field_order;
	__int64 limit;
	
	VIDEO_STREAM bs;
	SEQUENCE_HEADER seq;
	PICTURE_HEADER pic;
	READ_PICTURE_HEADER_OPTION pic_opt;

	GOP_LIST *gop_list;
	
	char gl_path[FILENAME_MAX];

	memset(&bs, 0, sizeof(bs));

	if (!is_registered_suffix(src_path)) {
		fprintf(stderr, "error - '%s' is unknown suffix.\n\t%s\n", read_suffix(src_path), src_path);
		goto LAST;
	}

	if (!open_video_stream(src_path, &bs)) {
		fprintf(stderr, "error - failed on open_video_stream().\n\t%s\n", src_path);
		goto LAST;
	}

	limit = get_filecheck_limit();
	limit += video_stream_tell(&bs);
	while (vs_next_start_code(&bs)) {
		code = vs_get_bits(&bs, 32);
		if (code == 0x1b3) {
			/* sequence start code */
			break;
		} else if (limit < video_stream_tell(&bs)) {
			/* file check limit exceeded */
			break;
		}
	}

	if (code != 0x1b3) {
		fprintf(stderr, "error - sequence_start_code is not found in %I64d bytes.\n\t%s", limit, src_path);
		goto LAST;
	}

	if (!read_sequence_header(&bs, &seq)) {
		fprintf(stderr, "error - failed on read_sequence_header().\n\t%s\n", src_path);
		goto LAST;
	}

	sequence_header_to_read_picture_header_option(&seq, &pic_opt);

	while(vs_next_start_code(&bs)){
		code = vs_get_bits(&bs, 32);
		if(code == 0x100){
			/* picture start code */
			break;
		}else if(limit < video_stream_tell(&bs)){
			/* file check limit exceeded */
			break;
		}
	}
	
	if (code != 0x100) {
		fprintf(stderr, "error - picture_start_code is not found in %I64d bytes.\n\t%s", limit, src_path);
		goto LAST;
	}

	if (!read_picture_header(&bs, &pic, &pic_opt)) {
		fprintf(stderr, "error - failed on read_picture_header().\n\t%s\n", src_path);
		goto LAST;
	}

	orig_field_order = pic.pc.top_field_first;

	gop_list = new_gop_list(&bs, &pic_opt, orig_field_order, GL_DIALOG_MODE_CUI);
	if (gop_list == NULL) {
		fprintf(stderr, "error - failed on new_gop_list().\n\t%s\n", src_path);
		goto LAST;
	}

	strcpy(gl_path, src_path);
	cut_suffix(gl_path);
	strcat(gl_path, ".gl");
	
	store_gop_list(gop_list, gl_path);
	
LAST:
	if (bs.fd >= 0) {
		close_video_stream(&bs);
		bs.fd = -1;
	}

	if (gop_list != NULL) {
		delete_gop_list(gop_list);
		gop_list = NULL;
	}
}

int main(int argc, char **argv)
{
	int i;

	if (argc < 2) {
		fprintf(stderr, "usage: make_gl input.file [more files]\n");
		exit(EXIT_FAILURE);
	}

	for (i=1;i<argc;i++) {
		make_gop_list(argv[i]);
	}

	return EXIT_SUCCESS;
}
